@@ -1291,7 +1291,6 @@ xref_meta(prolog_listen(Ev,G),  [G+N]) :- event_xargs(Ev, N).
 xref_meta(prolog_listen(Ev,G,_),[G+N]) :- event_xargs(Ev, N).
 xref_meta(tnot(G),		[G]).
 xref_meta(not_exists(G),	[G]).
-xref_meta(with_tty_raw(G),	[G]).
 
                                         % XPCE meta-predicates
 xref_meta(pce_global(_, new(_)), _) :- !, fail.
@@ -2225,8 +2224,7 @@ open_include_file(Path, In, [Ref]) :-
     stream_property(Parent, encoding(Enc)),
     '$push_input_context'(xref_include),
     catch((   prolog:xref_open_source(Path, In)
-          ->  catch(set_stream(In, encoding(Enc)),
-                    error(_,_), true)       % deal with non-file input
+          ->  set_stream(In, encoding(Enc))
           ;   include_encoding(Enc, Options),
               open(Path, read, In, Options)
           ), E,
