@@ -39,15 +39,20 @@
             concurrent_maplist/2,       % :Goal, +List
             concurrent_maplist/3,       % :Goal, ?List1, ?List2
             concurrent_maplist/4,       % :Goal, ?List1, ?List2, ?List3
+            concurrent_forall/2,        % :Generate, :Test
+            concurrent_forall/3,        % :Generate, :Test, +Options
+            concurrent_and/2,           % :Generator,:Test
+            concurrent_and/3,           % :Generator,:Test,+Options
             first_solution/3,           % -Var, :Goals, +Options
 
             call_in_thread/2            % +Thread, :Goal
           ]).
-:- use_module(library(debug)).
-:- use_module(library(error)).
-:- use_module(library(lists)).
-:- use_module(library(apply)).
-:- use_module(library(option)).
+:- autoload(library(apply),[maplist/2,maplist/3,maplist/4,maplist/5]).
+:- autoload(library(error),[must_be/2]).
+:- autoload(library(lists),[subtract/3,same_length/2]).
+:- autoload(library(option),[option/3]).
+:- autoload(library(ordsets), [ord_intersection/3]).
+:- autoload(library(debug), [debug/3, assertion/1]).
 
 %:- debug(concurrent).
 
@@ -56,6 +61,10 @@
     concurrent_maplist(1, +),
     concurrent_maplist(2, ?, ?),
     concurrent_maplist(3, ?, ?, ?),
+    concurrent_forall(0, 0),
+    concurrent_forall(0, 0, +),
+    concurrent_and(0, 0),
+    concurrent_and(0, 0, +),
     first_solution(-, :, +),
     call_in_thread(+, 0).
 
@@ -102,7 +111,7 @@ following consequences:
 @author Jan Wielemaker
 */
 
-%!  concurrent(+N, :Goals, Options) is semidet.
+%!  concurrent(+N, :Goals, +Options) is semidet.
 %
 %   Run Goals in parallel using N   threads.  This call blocks until
 %   all work has been done.  The   Goals  must  be independent. They
@@ -134,14 +143,14 @@ following consequences:
 %     * If one or more of the goals may fail or produce an error,
 %     using a higher number of threads may find this earlier.
 %
-%   @param N Number of worker-threads to create. Using 1, no threads
-%          are created.  If N is larger than the number of Goals we
-%          create exactly as many threads as there are Goals.
-%   @param Goals List of callable terms.
-%   @param Options Passed to thread_create/3 for creating the
-%          workers.  Only options changing the stack-sizes can
-%          be used. In particular, do not pass the detached or alias
-%          options.
+%   @arg N Number of worker-threads to create. Using 1, no threads
+%        are created.  If N is larger than the number of Goals we
+%        create exactly as many threads as there are Goals.
+%   @arg Goals List of callable terms.
+%   @arg Options Passed to thread_create/3 for creating the
+%        workers.  Only options changing the stack-sizes can
+%        be used. In particular, do not pass the detached or alias
+%        options.
 %   @see In many cases, concurrent_maplist/2 and friends
 %        is easier to program and is tractable to program
 %        analysis.
@@ -282,6 +291,248 @@ join_all([Id|T]) :-
     join_all(T).
 
 
+		 /*******************************
+		 *             FORALL		*
+		 *******************************/
+
+%!  concurrent_forall(:Generate, :Test) is semidet.
+%!  concurrent_forall(:Generate, :Test, +Options) is semidet.
+%
+%   True when Test is true for all   solutions of Generate. This has the
+%   same semantics as forall/2,  but  the   Test  goals  are executed in
+%   multiple threads. Notable a  failing  Test   or  a  Test throwing an
+%   exception signals the calling  thread  which   in  turn  aborts  all
+%   workers and fails or re-throws the generated error. Options:
+%
+%     - threads(+Count)
+%       Number of threads to use.  The default is determined by the
+%       Prolog flag `cpu_count`.
+%
+%   @tbd Ideally we would grow the   set of workers dynamically, similar
+%   to dynamic scheduling of  HTTP  worker   threads.  This  would avoid
+%   creating threads that are never used if Generate is too slow or does
+%   not provide enough answers and  would   further  raise the number of
+%   threads if Test is I/O bound rather than CPU bound.
+
+:- dynamic
+    fa_aborted/1.
+
+concurrent_forall(Generate, Test) :-
+    concurrent_forall(Generate, Test, []).
+
+concurrent_forall(Generate, Test, Options) :-
+    jobs(Jobs, Options),
+    Jobs > 1,
+    !,
+    term_variables(Generate, GVars),
+    term_variables(Test, TVars),
+    sort(GVars, GVarsS),
+    sort(TVars, TVarsS),
+    ord_intersection(GVarsS, TVarsS, Shared),
+    Templ =.. [v|Shared],
+    MaxSize is Jobs*4,
+    message_queue_create(Q, [max_size(MaxSize)]),
+    length(Workers, Jobs),
+    thread_self(Me),
+    maplist(thread_create(fa_worker(Q, Me, Templ, Test)), Workers),
+    catch(( forall(Generate,
+                   thread_send_message(Q, job(Templ))),
+            forall(between(1, Jobs, _),
+                   thread_send_message(Q, done)),
+            maplist(thread_join, Workers),
+            message_queue_destroy(Q)
+          ),
+          Error,
+          fa_cleanup(Error, Workers, Q)).
+concurrent_forall(Generate, Test, _) :-
+    forall(Generate, Test).
+
+fa_cleanup(Error, Workers, Q) :-
+    maplist(safe_abort, Workers),
+    debug(concurrent(fail), 'Joining workers', []),
+    maplist(safe_join, Workers),
+    debug(concurrent(fail), 'Destroying queue', []),
+    retractall(fa_aborted(Q)),
+    message_queue_destroy(Q),
+    (   Error = fa_worker_failed(Test, Why)
+    ->  debug(concurrent(fail), 'Test ~p failed: ~p', [Test, Why]),
+        (   Why == false
+        ->  fail
+        ;   Why = error(E)
+        ->  throw(E)
+        ;   assertion(fail)
+        )
+    ;   throw(Error)
+    ).
+
+fa_worker(Queue, Main, Templ, Test) :-
+    repeat,
+    thread_get_message(Queue, Msg),
+    (   Msg == done
+    ->  !
+    ;   Msg = job(Templ),
+        debug(concurrent, 'Running test ~p', [Test]),
+        (   catch_with_backtrace(Test, E, true)
+        ->  (   var(E)
+            ->  fail
+            ;   fa_stop(Queue, Main, fa_worker_failed(Test, error(E)))
+            )
+        ;   !,
+            fa_stop(Queue, Main, fa_worker_failed(Test, false))
+        )
+    ).
+
+fa_stop(Queue, Main, Why) :-
+    with_mutex('$concurrent_forall',
+               fa_stop_sync(Queue, Main, Why)).
+
+fa_stop_sync(Queue, _Main, _Why) :-
+    fa_aborted(Queue),
+    !.
+fa_stop_sync(Queue, Main, Why) :-
+    asserta(fa_aborted(Queue)),
+    debug(concurrent(fail), 'Stop due to ~p. Signalling ~q', [Why, Main]),
+    thread_signal(Main, throw(Why)).
+
+jobs(Jobs, Options) :-
+    (   option(threads(Jobs), Options)
+    ->  true
+    ;   current_prolog_flag(cpu_count, Jobs)
+    ->  true
+    ;   Jobs = 1
+    ).
+
+safe_abort(Thread) :-
+    catch(thread_signal(Thread, abort), error(_,_), true).
+safe_join(Thread) :-
+    E = error(_,_),
+    catch(thread_join(Thread, _Status), E, true).
+
+
+		 /*******************************
+		 *              AND		*
+		 *******************************/
+
+%!  concurrent_and(:Generator, :Test).
+%!  concurrent_and(:Generator, :Test, +Options).
+%
+%   Concurrent version of `(Generator,Test)`. This   predicate creates a
+%   thread providing solutions for Generator that   are handed to a pool
+%   of threads that run Test for   the different instantiations provided
+%   by Generator concurrently. The predicate  is logically equivalent to
+%   a simple conjunction except for two  aspects: (1) terms are _copied_
+%   from Generator to the test  Test   threads  while answers are copied
+%   back to the calling thread and (2)   answers  may be produced out of
+%   order.
+%
+%   If   the   evaluation   of   some    Test   raises   an   exception,
+%   concurrent_and/2,3 is terminated with this  exception. If the caller
+%   commits  after  a  given  answer  or    raises  an  exception  while
+%   concurrent_and/2,3  is  active  with  pending   choice  points,  all
+%   involved resources are reclaimed.
+%
+%   Options:
+%
+%     - threads(+Count)
+%       Create a worker pool holding Count threads.  The default is
+%       the Prolog flag `cpu_count`.
+%
+%   This    predicate    was    proposed     by      Jan     Burse    as
+%   balance((Generator,Test)).
+
+concurrent_and(Gen, Test) :-
+    concurrent_and(Gen, Test, []).
+
+concurrent_and(Gen, Test, Options) :-
+    jobs(Jobs, Options),
+    MaxSize is Jobs*4,
+    message_queue_create(JobQueue, [max_size(MaxSize)]),
+    message_queue_create(AnswerQueue),
+    ca_template(Gen, Test, Templ),
+    term_variables(Gen+Test, AllVars),
+    ReplyTempl =.. [v|AllVars],
+    length(Workers, Jobs),
+    Alive is 1<<Jobs-1,
+    maplist(thread_create(ca_worker(JobQueue, AnswerQueue,
+                                    Templ, Test, ReplyTempl)),
+            Workers),
+    thread_create(ca_generator(Gen, Templ, JobQueue, AnswerQueue),
+                  GenThread),
+    State = state(Alive),
+    call_cleanup(
+        ca_gather(State, AnswerQueue, ReplyTempl, Workers),
+        ca_cleanup(GenThread, Workers, JobQueue, AnswerQueue)).
+
+ca_gather(State, AnswerQueue, ReplyTempl, Workers) :-
+    repeat,
+       thread_get_message(AnswerQueue, Msg),
+       (   Msg = true(ReplyTempl)
+       ->  true
+       ;   Msg = done(Worker)
+       ->  nth0(Done, Workers, Worker),
+           arg(1, State, Alive0),
+           Alive1 is Alive0 /\ \(1<<Done),
+           debug(concurrent(and), 'Alive = ~2r', [Alive1]),
+           (   Alive1 =:= 0
+           ->  !,
+               fail
+           ;   nb_setarg(1, State, Alive1),
+               fail
+           )
+       ;   Msg = error(E)
+       ->  throw(E)
+       ).
+
+ca_template(Gen, Test, Templ) :-
+    term_variables(Gen,  GVars),
+    term_variables(Test, TVars),
+    sort(GVars, GVarsS),
+    sort(TVars, TVarsS),
+    ord_intersection(GVarsS, TVarsS, Shared),
+    ord_union(GVarsS, Shared, TemplVars),
+    Templ =.. [v|TemplVars].
+
+ca_worker(JobQueue, AnswerQueue, Templ, Test, ReplyTempl) :-
+    thread_self(Me),
+    EG = error(existence_error(message_queue, _), _),
+    repeat,
+    catch(thread_get_message(JobQueue, Req), EG, Req=all_done),
+    (   Req = job(Templ)
+    ->  (   catch(Test, E, true),
+            (   var(E)
+            ->  thread_send_message(AnswerQueue, true(ReplyTempl))
+            ;   thread_send_message(AnswerQueue, error(E))
+            ),
+            fail
+        )
+    ;   Req == done
+    ->  !,
+        message_queue_destroy(JobQueue),
+        thread_send_message(AnswerQueue, done(Me))
+    ;   assertion(Req == all_done)
+    ->  !,
+        thread_send_message(AnswerQueue, done(Me))
+    ).
+
+ca_generator(Gen, Templ, JobQueue, AnswerQueue) :-
+    (   catch(Gen, E, true),
+        (   var(E)
+        ->  thread_send_message(JobQueue, job(Templ))
+        ;   thread_send_message(AnswerQueue, error(E))
+        ),
+        fail
+    ;   thread_send_message(JobQueue, done)
+    ).
+
+ca_cleanup(GenThread, Workers, JobQueue, AnswerQueue) :-
+    safe_abort(GenThread),
+    safe_join(GenThread),
+    maplist(safe_abort, Workers),
+    maplist(safe_join, Workers),
+    message_queue_destroy(AnswerQueue),
+    catch(message_queue_destroy(JobQueue), error(_,_), true).
+
+
                  /*******************************
                  *             MAPLIST          *
                  *******************************/
