@@ -876,7 +876,7 @@ with one operation, it turns out to be faster as well.
 #define TRACE_ME		(0x02000000) /* Can be debugged */
 #define P_INCREMENTAL		(0x04000000) /* Incremental tabling */
 #define P_AUTOLOAD		(0x08000000) /* autoload/2 explicit import */
-#define P_TSHARED		(0x10000000) /* Using a shared table */
+#define P_WAITED_FOR		(0x10000000) /* Someone is waiting for this predicate */
 #define	P_LOCKED_SUPERVISOR	(0x20000000) /* Fixed supervisor */
 #define FILE_ASSIGNED		(0x40000000) /* Is assigned to a file */
 #define P_REDEFINED		(0x80000000) /* Overrules a definition */
@@ -927,6 +927,7 @@ with one operation, it turns out to be faster as well.
 #define UNKNOWN_MASK		(UNKNOWN_ERROR|UNKNOWN_WARNING|UNKNOWN_FAIL)
 #define M_VARPREFIX		(0x00008000) /* _var, Atom */
 #define M_DESTROYED		(0x00010000)
+#define M_WAITED_FOR		(0x00020000) /* thread_wait/2 on this module */
 
 /* Flags on functors */
 
@@ -1515,9 +1516,9 @@ typedef struct definition_refs
 #define PROC_IMPORTED	 (0x0004)	/* Procedure is imported */
 
 struct procedure
-{ Definition	 definition;		/* definition of procedure */
-  unsigned short flags;			/* PROC_WEAK */
-  unsigned short source_no;		/* Source I'm assigned to */
+{ Definition	definition;		/* definition of procedure */
+  unsigned int  flags;			/* PROC_WEAK */
+  unsigned int	source_no;		/* Source I'm assigned to */
 };
 
 struct localFrame
@@ -1771,8 +1772,9 @@ struct sourceFile
 #endif
   int		magic;			/* Magic number */
   int		count;			/* number of times loaded */
-  unsigned	number_of_clauses;	/* number of clauses */
-  unsigned	index     : 24;		/* index number (1,2,...) */
+  unsigned int	number_of_clauses;	/* number of clauses */
+  unsigned int	index;			/* index number (1,2,...) */
+  unsigned int	references;		/* Reference count */
   unsigned	system     : 1;		/* system sourcefile: do not reload */
   unsigned	from_state : 1;		/* Loaded from resource DB state */
   unsigned	resource   : 1;		/* Loaded from resource DB file */
@@ -1805,6 +1807,7 @@ struct module
   size_t	code_limit;	/* Limit for code_size */
 #ifdef O_PLMT
   counting_mutex *mutex;	/* Mutex to guard module modifications */
+  struct thread_wait_area *wait;/* Manage waiting threads */
 #endif
 #ifdef O_PROLOG_HOOK
   Procedure	hook;		/* Hooked module */
@@ -1816,6 +1819,19 @@ struct module
   gen_t		last_modified;	/* Generation I was last modified */
 };
 
+#define MENUM_TEMP	0x1	/* Also enumerate temporary modules */
+
+typedef struct module_enum
+{ TableEnum	tenum;
+  Module        current;
+  int		flags;
+} module_enum, *ModuleEnum;
+
+
+		 /*******************************
+		 *	      TRAIL		*
+		 *******************************/
+
 struct trail_entry
 { Word		address;	/* address of the variable */
 };
