@@ -36,6 +36,7 @@
 
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
+#include "pl-comp.h"
 #include "pl-arith.h"
 #include "pl-dbref.h"
 #include "pl-termwalk.c"
@@ -601,7 +602,8 @@ compile_term_to_heap(term_agenda *agenda, CompileInfo info ARG_LD)
 
 	  mark.term = f;
 	  mark.fdef = f->definition;
-	  pushSegStack(&LD->cycle.lstack, mark, cycle_mark);
+	  if ( !pushSegStack(&LD->cycle.lstack, mark, cycle_mark) )
+	    return FALSE;
 	  f->definition = (functor_t)consUInt(info->size);
 				  /* overflow test (should not be possible) */
 	  DEBUG(CHK_SECURE, assert(valUInt(f->definition) == (uintptr_t)info->size));
@@ -615,7 +617,8 @@ compile_term_to_heap(term_agenda *agenda, CompileInfo info ARG_LD)
 		   Sdprintf("Added %s/%d\n",
 			    stringAtom(valueFunctor(functor)->name),
 			    arityFunctor(functor)));
-	pushWorkAgenda(agenda, arity, f->arguments);
+	if ( !pushWorkAgenda(agenda, arity, f->arguments) )
+	  return FALSE;
 	continue;
       }
       default:
@@ -686,6 +689,7 @@ compileTermToHeap__LD(term_t t,
   size_t size;
   size_t rsize = SIZERECORD(flags);
   term_agenda agenda;
+  int rc;
 
   DEBUG(CHK_SECURE, checkData(valTermRef(t)));
 
@@ -698,30 +702,34 @@ compileTermToHeap__LD(term_t t,
   info.lock = !(info.external || (flags&R_NOLOCK));
 
   initTermAgenda(&agenda, 1, valTermRef(t));
-  compile_term_to_heap(&agenda, &info PASS_LD);
+  rc = compile_term_to_heap(&agenda, &info PASS_LD);
   clearTermAgenda(&agenda);
   restoreVars(&info);
   unvisit(PASS_LD1);
 
-  size = rsize + sizeOfBuffer(&info.code);
-  if ( allocate )
-    record = (*allocate)(closure, size);
-  else
-    record = PL_malloc_atomic_unmanaged(size);
+  if ( rc )
+  { size = rsize + sizeOfBuffer(&info.code);
+    if ( allocate )
+      record = (*allocate)(closure, size);
+    else
+      record = PL_malloc_atomic_unmanaged(size);
 
-  if ( record )
-  {
+    if ( record )
+    {
 #ifdef REC_MAGIC
-    record->magic = REC_MAGIC;
+      record->magic = REC_MAGIC;
 #endif
-    record->gsize = (unsigned int)info.size; /* only 28-bit */
-    record->nvars = info.nvars;
-    record->size  = (int)size;
-    record->flags = flags;
-    if ( flags & R_DUPLICATE )
-    { record->references = 1;
+      record->gsize = (unsigned int)info.size; /* only 28-bit */
+      record->nvars = info.nvars;
+      record->size  = (int)size;
+      record->flags = flags;
+      if ( flags & R_DUPLICATE )
+      { record->references = 1;
+      }
+      memcpy(addPointer(record, rsize), info.code.base, sizeOfBuffer(&info.code));
     }
-    memcpy(addPointer(record, rsize), info.code.base, sizeOfBuffer(&info.code));
+  } else
+  { record = NULL;
   }
   discardBuffer(&info.code);
 
@@ -750,7 +758,6 @@ variantRecords(const Record r1, const Record r2)
 #define	REC_GROUND  0x10		/* Record is ground */
 #define	REC_VMASK   0xe0		/* Version mask */
 #define REC_VSHIFT     5		/* shift for version mask */
-#define	REC_VERSION 0x03		/* Version id */
 
 #define REC_SZMASK  (REC_32|REC_64)	/* SIZE_MASK */
 
@@ -760,7 +767,7 @@ variantRecords(const Record r1, const Record r2)
 #define REC_SZ REC_32
 #endif
 
-#define REC_HDR		(REC_SZ|(REC_VERSION<<REC_VSHIFT))
+#define REC_HDR		(REC_SZ|(PL_REC_VERSION<<REC_VSHIFT))
 #define REC_COMPAT(m)	(((m)&(REC_VMASK|REC_SZMASK)) == REC_HDR)
 
 typedef struct record_data
@@ -960,7 +967,7 @@ PRED_IMPL("fast_term_serialized", 2, fast_term_serialized, 0)
     { return FALSE;
     }
   } else if ( PL_get_nchars(string, &len, &rec,
-			    CVT_STRING|BUF_RING|REP_ISO_LATIN_1|CVT_EXCEPTION) )
+			    CVT_STRING|BUF_STACK|REP_ISO_LATIN_1|CVT_EXCEPTION) )
   { term_t tmp;
 
     return ( (tmp = PL_new_term_ref()) &&
@@ -1844,7 +1851,7 @@ PL_recorded_external(const char *rec, term_t t)
       if ( !b.version_map )
       { Sdprintf("PL_recorded_external(): "
 		 "Incompatible version (%d, current %d)\n",
-		 save_version, REC_VERSION);
+		 save_version, PL_REC_VERSION);
 	fail;
       }
     }
@@ -2367,7 +2374,7 @@ PRED_IMPL("erase", 1, erase, 0)
       return PL_error("erase", 1, NULL, ERR_PERMISSION,
 		      ATOM_clause, ATOM_erase, ref);
 
-    return retractClauseDefinition(def, clause);
+    return retractClauseDefinition(def, clause, TRUE);
   } else
   { RecordRef r = ptr;
     int rc;
